# Excel data
    df = pd.read_excel('{{ dataset_path }}')
    print(f"Loaded {len(df)} records from Excel file")

    # Optional: specify columns if needed
    # df = pd.read_excel('{{ dataset_path }}', usecols=['col1', 'col2', 'col3'])