# TSV data
    df = pd.read_csv('{{ dataset_path }}', sep='\t', encoding='utf-8')
    print(f"Loaded {len(df)} records from TSV file")

    # Optional: specify columns if needed
    # df = pd.read_csv('{{ dataset_path }}', sep='\t', usecols=['col1', 'col2', 'col3'], encoding='utf-8')