[project]
name = "{{ project_name }}"
version = "0.1.0"
description = "MATER project (generated by init-mater-project)"
readme = "README.md"
requires-python = ">=3.12"
dependencies = [
    "click>=8.0.0",
    "mater>=0.12.3",
    "pydantic-settings>=2.10.1",
    "pydantic[email]>=2.11.7",
    "rich-click>=1.8.9",
]

[project.urls]
"Generator" = "https://gricad-gitlab.univ-grenoble-alpes.fr/isterre-dynamic-modeling/mater-project/init-mater-project"
"Documentation" = "https://isterre-dynamic-modeling.gricad-pages.univ-grenoble-alpes.fr/mater-project/init-mater-project"

[project.scripts]
mater-cli = "cli:cli"

[tool.setuptools]
py-modules = ["cli", "settings"]  
packages = ["commands"]  

[build-system]
requires = ["setuptools>=61.0", "wheel"]
build-backend = "setuptools.build_meta"
