# MATER Simulation Configuration
# Edit this file to customize your simulation parameters

[simulation]
name = "simulation_outputs"
start_time = 1900
end_time = 2100
frequency = "YS"
scenario = "historical"

[paths]
dimensions_hierarchy_file = "data/dimensions_hierarchy/dimensions_hierarchy.json"
dimensions_mapping_file = "data/dimensions_hierarchy/dimensions_mapping.json"
dimensions_values_path = "data/references/dimensions.json"
input_path = "data/input"
output_folder = "outputs"
raw_path = "data/raw"
transforms_script_path = "transforms/datasets"
variables_dimensions_path = "data/references/variables_dimensions.json"

[provider]
first_name = "Lauranne"
last_name = "Sarribouette"
email = "lauranne.sarribouette@univ-grenoble-alpes.fr"

[metadata]
source = "Example Dataset Source"
link = "https://example.org/dataset"
project = "Example MATER Project" 